#include <StringView.h>
#include <stdio.h>
#include <Bitmap.h>
#include <File.h>
#include <Resources.h>
#include <Slider.h>
#include <stdlib.h>
#include <TranslationUtils.h>
#include <Screen.h>
#include "KITT.h"

// MAIN INSTANTIATION FUNCTION
extern "C" _EXPORT BScreenSaver *instantiate_screen_saver(BMessage *message, image_id image)
{
	return new KITT(message, image);
}


// Constructor and Deconstructor
KITT::KITT(BMessage *message, image_id image)
 : BScreenSaver(message, image)
{
   	image_info info;
    BResources resources;
	if (get_image_info(image,&info) == B_OK) {
		BFile file (info.name, B_READ_ONLY);
	    if(resources.SetTo(&file) == B_OK){
//	    	printf("KITT:KITT() resource file found.\n");
	    }
	}

	for (int i=0; i<nrBlocks; i++)
	{
	   	// Try to load a logo bitmap from the add-on's resources.
//		bool test;
//	    test = resources.HasResource(B_RAW_TYPE, 100+i);
//	    if(test){
//			printf("KITT:KITT() resource BLOCK_%i.BMP exists.\n", i);
//	    }

	    size_t size;
  		const void *pData = resources.LoadResource(B_RAW_TYPE, 100+i, &size);

  		if (pData) {
//			printf("KITT:KITT() loaded resource BLOCK_%i.BMP.\n", i);
      		BMemoryIO stream(pData, size);
       		m_bitmap[i] = BTranslationUtils::GetBitmap(&stream);
   		}
	
		m_blockState[i] = 0;
	}
//	printf("KITT:KITT() end constructor.\n");
}

KITT::~KITT(void)
{
	for (int i=0; i<nrBlocks; i++)
	{
		if (m_bitmap[i]) {
			delete m_bitmap[i];
		}
	}
}

//********************************************************                   
//	The BView shown in the screensaver-preference window
//********************************************************
void 
KITT::StartConfig(BView *view)
{	
	BView *tView;
	
	view->AddChild(tView=new BStringView(BRect(8, 0, 200, 20), B_EMPTY_STRING, "Knight Industries Two Thousand"));
	tView->SetFontSize(12);
	view->AddChild(new BStringView(BRect(70, 70, 200, 80), B_EMPTY_STRING, ".......Michael, watch out!"));
	view->AddChild(new BStringView(BRect(10, 175, 200, 190), B_EMPTY_STRING, "Rogier van der Hee, 2004"));
	view->AddChild(new BStringView(BRect(10, 200, 200, 215), B_EMPTY_STRING, "(And a whole lot of other people)"));
}


//********************************************************
//	initialize screensaver
//********************************************************
status_t 
KITT::StartSaver(BView *view, bool preview)
{
	SetTickSize(180000); // Rate at which Draw function is called
    
	// Get width and height bitmap (assuming all are same size!)
	float height = 0.0f; 
	if (m_bitmap[1] != NULL)
	{
		height = m_bitmap[1]->Bounds().Height(); 
		m_width = m_bitmap[1]->Bounds().Width(); 
	} 
//	printf("Width bitmap: %.2f\n", m_width);
	
	// Find center of screen and set member variables y and offset
	BScreen screen;
	m_y = (screen.Frame().Height() - height)/2.0f;	// use real 32bit float to avoid type casting from double
	m_offset = (screen.Frame().Width() - nrBlocks * m_width)/2.0f; // calc center

	m_brightestBlock = 0;
	m_direction = leftToRight;

	return B_OK;
}

//********************************************************
//	Arrange the blocks
//********************************************************
//void	
//KITT::ArrangeBlocks(BView *view){
	
	// initialize 2 arrays	
	
	
	
//}


//********************************************************
//	Draw the bitmaps to the screen
//********************************************************
void 
KITT::Draw(BView *view, int32 frame)
{
//    printf("KITT:Draw() called.\n");
	// Clear the screen
	if (frame==0) { 
		view->SetHighColor(0,0,0); 
		view->FillRect(view->Bounds());
	}

	//printf("m_brightestBlock %i\n", m_brightestBlock);

    // fade (decrement) all blocks
    for (int i = 0; i < nrBlocks; i++) {
    	if (m_blockState[i] > 0) {
    		m_blockState[i]--;
    	}
    }
    
    // light up the brightest block (brightest bitmap is nrBlocks - 1)
    m_blockState[m_brightestBlock] = nrBlocks - 1;
	
	// draw the blocks
	for (int i=0; i < nrBlocks; i++) {
	    //printf("drawing %i, blockState %i\n", i, m_blockState[i]); 
		if (m_bitmap[m_blockState[i]] != NULL){
		    // XXX: Fix the bitmap order (and remove nrBlocks - m_blockState[i])!
			view->DrawBitmap( m_bitmap[nrBlocks - m_blockState[i] - 1], BPoint(i*m_width + m_offset, m_y ) );
		}
	}	

    // next time light up next block
    m_brightestBlock += m_direction;

    // change the direction
	if (m_brightestBlock == 0) {
		m_direction = leftToRight;
   }
	if (m_brightestBlock == nrBlocks-1) {
		m_direction = rightToLeft;

	}

}
